@@ -31,11 +31,10 @@ VarDecl(..),
 -- * Declaration attributes
 DeclAttrs(..),isExtDecl,
 Storage(..),declStorage,ThreadLocal,Register,
-Linkage(..),
+Linkage(..),hasLinkage,declLinkage,
 -- * Types
 Type(..),
-FunType(..),isFunctionType,
-derefTypeDef,referencedType,hasTypeOfExpr,
+FunType(..),
 ArraySize(..),
 TypeDefRef(..),
 TypeName(..),BuiltinType(..),
@@ -46,9 +45,9 @@ EnumTypeRef(..),EnumType(..),typeOfEnumDef,
 Enumerator(..),
 TypeQuals(..),noTypeQuals,mergeTypeQuals,
 -- * Variable names
-VarName(..),identOfVarName,AsmName,
+VarName(..),identOfVarName,isNoName,AsmName,
 -- * Attributes (STUB, not yet analyzed)
-Attr(..),Attributes,
+Attr(..),Attributes,noAttributes,mergeAttributes,
 -- * Statements and Expressions (STUB, aliases to Syntax)
 Stmt,Expr,Initializer,AsmBlock,
 )
@@ -73,7 +72,7 @@ class HasCompTyKind a where
 -- | Composite type definitions (tags)
 data TagDef =  CompDef CompType	  --definition
      	       | EnumDef EnumType      -- enum definition
-               deriving (Typeable, Data {-! CNode !-})
+               deriving (Typeable, Data {-! ,CNode !-})
 
 instance HasSUERef TagDef where
     sueRef (CompDef ct) = sueRef ct
@@ -115,7 +114,7 @@ data IdentDecl = Declaration Decl           -- ^ object or function declaration
 	             | ObjectDef ObjDef           -- ^ object definition
 	             | FunctionDef FunDef         -- ^ function definition
 	             | EnumeratorDef Enumerator   -- ^ definition of an enumerator
-               deriving (Typeable, Data {-! CNode !-})
+               deriving (Typeable, Data {-! ,CNode !-})
 
 instance Declaration IdentDecl where
   getVarDecl (Declaration decl) = getVarDecl decl
@@ -190,6 +189,10 @@ data DeclEvent =
        -- ^ file-scope struct\/union\/enum event
      | DeclEvent IdentDecl
        -- ^ file-scope declaration or definition
+     | ParamEvent ParamDecl
+       -- ^ parameter declaration
+     | LocalEvent IdentDecl
+       -- ^ local variable declaration or definition
      | TypeDefEvent TypeDef
        -- ^ a type definition
      | AsmEvent AsmBlock
@@ -200,7 +203,7 @@ data DeclEvent =
 
 -- | Declarations, which aren't definitions
 data Decl = Decl VarDecl NodeInfo
-            deriving (Typeable, Data {-! CNode !-})
+            deriving (Typeable, Data {-! ,CNode !-})
 
 instance Declaration Decl where
     getVarDecl   (Decl vd _) =  vd
@@ -212,7 +215,7 @@ instance Declaration Decl where
 -- If the initializer is missing, it is a tentative definition, i.e. a
 -- definition which might be overriden later on.
 data ObjDef = ObjDef VarDecl (Maybe Initializer) NodeInfo
-             deriving (Typeable, Data {-! CNode !-})
+             deriving (Typeable, Data {-! ,CNode !-})
 instance Declaration ObjDef where
     getVarDecl  (ObjDef vd _ _) =  vd
 
@@ -225,7 +228,7 @@ isTentative (ObjDef decl init_opt _) | isExtDecl decl = maybe True (const False)
 --
 -- A function definition is a declaration together with a statement (the function body).
 data FunDef = FunDef VarDecl Stmt NodeInfo
-             deriving (Typeable, Data {-! CNode !-})
+             deriving (Typeable, Data {-! ,CNode !-})
 instance Declaration FunDef where
     getVarDecl (FunDef vd _ _) = vd
 
@@ -233,7 +236,7 @@ instance Declaration FunDef where
 -- | Parameter declaration
 data ParamDecl = ParamDecl VarDecl NodeInfo
                | AbstractParamDecl VarDecl NodeInfo
-    deriving (Typeable, Data {-! CNode !-} )
+    deriving (Typeable, Data {-! ,CNode !-} )
 
 instance Declaration ParamDecl where
   getVarDecl (ParamDecl vd _) = vd
@@ -244,7 +247,7 @@ data MemberDecl = MemberDecl VarDecl (Maybe Expr) NodeInfo
                   -- ^ @MemberDecl vardecl bitfieldsize node@
                 | AnonBitField Type Expr NodeInfo
                   -- ^ @AnonBitField typ size@
-    deriving (Typeable, Data {-! CNode !-} )
+    deriving (Typeable, Data {-! ,CNode !-} )
 
 instance Declaration MemberDecl where
   getVarDecl (MemberDecl vd _ _) = vd
@@ -254,7 +257,7 @@ instance Declaration MemberDecl where
 --
 -- The identifier is a new name for the given type.
 data TypeDef = TypeDef Ident Type Attributes NodeInfo
-               deriving (Typeable, Data {-! CNode !-} )
+               deriving (Typeable, Data {-! ,CNode !-} )
 
 -- | return the idenitifier of a @typedef@
 identOfTypeDef :: TypeDef -> Ident
@@ -297,76 +300,56 @@ declStorage d = case declAttrs d of (DeclAttrs _ st _) -> st
 -- | Storage duration and linkage of a variable
 data Storage  =  NoStorage                  -- ^ no storage
                | Auto Register              -- ^ automatic storage (optional: register)
-               | Static Linkage ThreadLocal -- ^ static storage, with linkage and thread local specifier (gnu c)
+               | Static Linkage ThreadLocal -- ^ static storage, linkage spec and thread local specifier (gnu c)
                | FunLinkage Linkage         -- ^ function, either internal or external linkage
                deriving (Typeable, Data, Show, Eq, Ord)
 
 type ThreadLocal = Bool
 type Register    = Bool
 
--- | Linkage: Either internal to the translation unit or external
-data Linkage = InternalLinkage | ExternalLinkage
+-- | Linkage: Either no linkage, internal to the translation unit or external
+data Linkage = NoLinkage | InternalLinkage | ExternalLinkage
                deriving (Typeable, Data, Show, Eq, Ord)
 
 -- | return @True@ if the object has linkage
 hasLinkage :: Storage -> Bool
-hasLinkage (Static _ _) = True
-hasLinkage _ = False
+hasLinkage (Auto _) = False
+hasLinkage (Static NoLinkage _) = False
+hasLinkage _ = True
+
+-- | Get the linkage of a definition
+declLinkage :: (Declaration d) => d -> Linkage
+declLinkage decl =
+    case declStorage decl of
+        NoStorage -> undefined
+        Auto _ -> NoLinkage
+        Static linkage _ -> linkage
+        FunLinkage linkage -> linkage
+
 
 -- * types
 
 -- | types of C objects
 data Type =
-       DirectType TypeName TypeQuals
+       DirectType TypeName TypeQuals Attributes
      -- ^ a non-derived type
      | PtrType Type TypeQuals Attributes
      -- ^ pointer type
      | ArrayType Type ArraySize TypeQuals Attributes
      -- ^ array type
-     | FunctionType FunType
+     | FunctionType FunType Attributes
      -- ^ function type
-     | TypeDefType TypeDefRef
+     | TypeDefType TypeDefRef TypeQuals Attributes
      -- ^ a defined type
-     | TypeOfExpr Expr
-     -- ^ (GNU) typeof (/broken/ and should be removed, but we do not yet have expression type analysis)
      deriving (Typeable, Data)
 
--- | Function types are of the form @FunType return-type params isVariadic attrs@.
+-- | Function types are of the form @FunType return-type params isVariadic@.
 --
 -- If the parameter types aren't yet known, the function has type @FunTypeIncomplete type attrs@.
-data FunType = FunType Type [ParamDecl] Bool Attributes
-            |  FunTypeIncomplete Type Attributes
+data FunType = FunType Type [ParamDecl] Bool
+            |  FunTypeIncomplete Type
                deriving (Typeable, Data)
 
--- | resolve typedefs, if possible
-derefTypeDef :: Type -> Type
-derefTypeDef (TypeDefType (TypeDefRef _ (Just actual_ty) _)) = derefTypeDef actual_ty
-derefTypeDef ty = ty
-
--- may not be called on undefined typeDefs or typeof(expr)
-referencedType :: Type -> Maybe Type
-referencedType (PtrType ty _ _) = Just ty
-referencedType (FunctionType (FunType ty _ _ _)) = Just ty
-referencedType (ArrayType ty _ _ _) = Just ty
-referencedType (TypeDefType (TypeDefRef _ (Just actual_ty) _)) = Just actual_ty
-referencedType (DirectType _ _) = Nothing
-referencedType _ = error "referencedType: failed to resolve type"
-
-hasTypeOfExpr :: Type -> Bool
-hasTypeOfExpr (TypeOfExpr _) = True
-hasTypeOfExpr ty = maybe False hasTypeOfExpr (referencedType ty)
-
--- | return @True@ if the given type is a function type
---
---   Result is undefined in the presence of TypeOfExpr or undefined typeDefs
-isFunctionType :: Type -> Bool
-isFunctionType ty =
-    case ty of  TypeDefType (TypeDefRef _ (Just actual_ty) _) -> isFunctionType actual_ty
-                TypeDefType _ -> error "isFunctionType: unresolved typeDef"
-                TypeOfExpr _  -> error "isFunctionType: typeof(expr)"
-                FunctionType _ -> True
-                _ -> False
-
 -- | An array type may either have unknown size or a specified array size, the latter either variable or constant.
 -- Furthermore, when used as a function parameters, the size may be qualified as /static/.
 -- In a function prototype, the size may be `Unspecified variable size' (@[*]@).
@@ -387,14 +370,15 @@ data TypeName =
     | TyBuiltin BuiltinType
     deriving (Typeable, Data)
 
--- | Builtin type (va_list)
+-- | Builtin type (va_list, anything)
 data BuiltinType = TyVaList
+                 | TyAny
                    deriving (Typeable, Data)
 
 -- | typdef references
 -- If the actual type is known, it is attached for convenience
 data TypeDefRef = TypeDefRef Ident (Maybe Type) NodeInfo
-               deriving (Typeable, Data {-! CNode !-})
+               deriving (Typeable, Data {-! ,CNode !-})
 
 -- | integral types (C99 6.7.2.2)
 data IntType =
@@ -440,24 +424,24 @@ instance Show FloatType where
 
 -- | composite type declarations
 data CompTypeRef = CompTypeRef SUERef CompTyKind NodeInfo
-                    deriving (Typeable, Data {-! CNode !-})
+                    deriving (Typeable, Data {-! ,CNode !-})
 
 instance HasSUERef  CompTypeRef where sueRef  (CompTypeRef ref _ _) = ref
 instance HasCompTyKind CompTypeRef where compTag (CompTypeRef _ tag _)  = tag
 
 data EnumTypeRef = EnumTypeRef SUERef NodeInfo
-    deriving (Typeable, Data {-! CNode !-})
+    deriving (Typeable, Data {-! ,CNode !-})
 instance HasSUERef  EnumTypeRef where sueRef  (EnumTypeRef ref _) = ref
 
 -- | Composite type (struct or union).
 data CompType =  CompType SUERef CompTyKind [MemberDecl] Attributes NodeInfo
-                 deriving (Typeable, Data {-! CNode !-} )
+                 deriving (Typeable, Data {-! ,CNode !-} )
 instance HasSUERef  CompType where sueRef  (CompType ref _ _ _ _) = ref
 instance HasCompTyKind CompType where compTag (CompType _ tag _ _ _) = tag
 
 -- | return the type of a composite type definition
 typeOfCompDef :: CompType -> TypeName
-typeOfCompDef (CompType ref tag _ _ _) = TyComp (CompTypeRef ref tag internalNode)
+typeOfCompDef (CompType ref tag _ _ _) = TyComp (CompTypeRef ref tag undefNode)
 
 -- | a tag to determine wheter we refer to a @struct@ or @union@, see 'CompType'.
 data CompTyKind =  StructTag
@@ -471,23 +455,23 @@ instance Show CompTyKind where
 -- | Representation of C enumeration types
 data EnumType = EnumType SUERef [Enumerator] Attributes NodeInfo
                  -- ^ @EnumType name enumeration-constants attrs node@
-                 deriving (Typeable, Data {-! CNode !-} )
+                 deriving (Typeable, Data {-! ,CNode !-} )
 
 instance HasSUERef EnumType where sueRef  (EnumType ref _ _ _) = ref
 
 -- | return the type of an enum definition
 typeOfEnumDef :: EnumType -> TypeName
-typeOfEnumDef (EnumType ref _ _ _) = TyEnum (EnumTypeRef ref internalNode)
+typeOfEnumDef (EnumType ref _ _ _) = TyEnum (EnumTypeRef ref undefNode)
 
 -- | An Enumerator consists of an identifier, a constant expressions and the link to its type
 data Enumerator = Enumerator Ident Expr EnumType NodeInfo
-                  deriving (Typeable, Data {-! CNode !-})
+                  deriving (Typeable, Data {-! ,CNode !-})
 instance Declaration Enumerator where
   getVarDecl (Enumerator ide _ enumty _) =
     VarDecl
       (VarName ide Nothing)
       (DeclAttrs False NoStorage [])
-      (DirectType (typeOfEnumDef enumty) noTypeQuals)
+      (DirectType (typeOfEnumDef enumty) noTypeQuals noAttributes)
 
 -- | Type qualifiers: constant, volatile and restrict
 data TypeQuals = TypeQuals { constant :: Bool, volatile :: Bool, restrict :: Bool }
@@ -526,6 +510,10 @@ identOfVarName :: VarName -> Ident
 identOfVarName NoName            = error "identOfVarName: NoName"
 identOfVarName (VarName ident _) = ident
 
+isNoName :: VarName -> Bool
+isNoName NoName = True
+isNoName _ = False
+
 -- | Top level assembler block (alias for @CStrLit@)
 type AsmBlock = CStrLit
 
@@ -551,109 +539,147 @@ type AsmName = CStrLit
 --
 -- /TODO/: ultimatively, we want to parse attributes and represent them in a typed way
 data Attr = Attr Ident [Expr] NodeInfo
-            deriving (Typeable, Data {-! CNode !-})
+            deriving (Typeable, Data {-! ,CNode !-})
 
 type Attributes = [Attr]
 
+-- |Empty attribute list
+noAttributes :: Attributes
+noAttributes = []
+
+-- |Merge attribute lists
+-- /TODO/: currently does not remove duplicates
+mergeAttributes :: Attributes -> Attributes -> Attributes
+mergeAttributes = (++)
+
 -- * statements and expressions (Type aliases)
 
 -- | 'Stmt' is an alias for 'CStat' (Syntax)
 type Stmt = CStat
 -- | 'Expr' is currently an alias for 'CExpr' (Syntax)
 type Expr = CExpr
+-- GENERATED START
 
 
+instance CNode TagDef where
+        nodeInfo (CompDef d) = nodeInfo d
+        nodeInfo (EnumDef d) = nodeInfo d
 
---------------------------------------------------------
--- DERIVES GENERATED CODE
--- DO NOT MODIFY BELOW THIS LINE
--- CHECKSUM: 1672142450
+instance Pos TagDef where
+        posOf x = posOf (nodeInfo x)
 
-instance CNode TagDef
-    where nodeInfo (CompDef d) = nodeInfo d
-          nodeInfo (EnumDef d) = nodeInfo d
-instance Pos TagDef
-    where posOf x = posOfNode (nodeInfo x)
 
-instance CNode IdentDecl
-    where nodeInfo (Declaration d) = nodeInfo d
-          nodeInfo (ObjectDef d) = nodeInfo d
-          nodeInfo (FunctionDef d) = nodeInfo d
-          nodeInfo (EnumeratorDef d) = nodeInfo d
-instance Pos IdentDecl
-    where posOf x = posOfNode (nodeInfo x)
+instance CNode IdentDecl where
+        nodeInfo (Declaration d) = nodeInfo d
+        nodeInfo (ObjectDef d) = nodeInfo d
+        nodeInfo (FunctionDef d) = nodeInfo d
+        nodeInfo (EnumeratorDef d) = nodeInfo d
 
-instance CNode DeclEvent
-    where nodeInfo (TagEvent d) = nodeInfo d
-          nodeInfo (DeclEvent d) = nodeInfo d
-          nodeInfo (TypeDefEvent d) = nodeInfo d
-          nodeInfo (AsmEvent d) = nodeInfo d
-instance Pos DeclEvent
-    where posOf x = posOfNode (nodeInfo x)
+instance Pos IdentDecl where
+        posOf x = posOf (nodeInfo x)
 
-instance CNode Decl
-    where nodeInfo (Decl _ nodeinfo) = nodeinfo
-instance Pos Decl
-    where posOf x = posOfNode (nodeInfo x)
 
-instance CNode ObjDef
-    where nodeInfo (ObjDef _ _ nodeinfo) = nodeinfo
-instance Pos ObjDef
-    where posOf x = posOfNode (nodeInfo x)
+instance CNode DeclEvent where
+        nodeInfo (TagEvent d) = nodeInfo d
+        nodeInfo (DeclEvent d) = nodeInfo d
+        nodeInfo (ParamEvent d) = nodeInfo d
+        nodeInfo (LocalEvent d) = nodeInfo d
+        nodeInfo (TypeDefEvent d) = nodeInfo d
+        nodeInfo (AsmEvent d) = nodeInfo d
 
-instance CNode FunDef
-    where nodeInfo (FunDef _ _ nodeinfo) = nodeinfo
-instance Pos FunDef
-    where posOf x = posOfNode (nodeInfo x)
+instance Pos DeclEvent where
+        posOf x = posOf (nodeInfo x)
 
-instance CNode ParamDecl
-    where nodeInfo (ParamDecl _ nodeinfo) = nodeinfo
-          nodeInfo (AbstractParamDecl _ nodeinfo) = nodeinfo
-instance Pos ParamDecl
-    where posOf x = posOfNode (nodeInfo x)
 
-instance CNode MemberDecl
-    where nodeInfo (MemberDecl _ _ nodeinfo) = nodeinfo
-          nodeInfo (AnonBitField _ _ nodeinfo) = nodeinfo
-instance Pos MemberDecl
-    where posOf x = posOfNode (nodeInfo x)
+instance CNode Decl where
+        nodeInfo (Decl _ n) = n
 
-instance CNode TypeDef
-    where nodeInfo (TypeDef _ _ _ nodeinfo) = nodeinfo
-instance Pos TypeDef
-    where posOf x = posOfNode (nodeInfo x)
+instance Pos Decl where
+        posOf x = posOf (nodeInfo x)
 
-instance CNode TypeDefRef
-    where nodeInfo (TypeDefRef _ _ nodeinfo) = nodeinfo
-instance Pos TypeDefRef
-    where posOf x = posOfNode (nodeInfo x)
 
-instance CNode CompTypeRef
-    where nodeInfo (CompTypeRef _ _ nodeinfo) = nodeinfo
-instance Pos CompTypeRef
-    where posOf x = posOfNode (nodeInfo x)
+instance CNode ObjDef where
+        nodeInfo (ObjDef _ _ n) = n
 
-instance CNode EnumTypeRef
-    where nodeInfo (EnumTypeRef _ nodeinfo) = nodeinfo
-instance Pos EnumTypeRef
-    where posOf x = posOfNode (nodeInfo x)
+instance Pos ObjDef where
+        posOf x = posOf (nodeInfo x)
 
-instance CNode CompType
-    where nodeInfo (CompType _ _ _ _ nodeinfo) = nodeinfo
-instance Pos CompType
-    where posOf x = posOfNode (nodeInfo x)
 
-instance CNode EnumType
-    where nodeInfo (EnumType _ _ _ nodeinfo) = nodeinfo
-instance Pos EnumType
-    where posOf x = posOfNode (nodeInfo x)
+instance CNode FunDef where
+        nodeInfo (FunDef _ _ n) = n
 
-instance CNode Enumerator
-    where nodeInfo (Enumerator _ _ _ nodeinfo) = nodeinfo
-instance Pos Enumerator
-    where posOf x = posOfNode (nodeInfo x)
+instance Pos FunDef where
+        posOf x = posOf (nodeInfo x)
 
-instance CNode Attr
-    where nodeInfo (Attr _ _ nodeinfo) = nodeinfo
-instance Pos Attr
-    where posOf x = posOfNode (nodeInfo x)
+
+instance CNode ParamDecl where
+        nodeInfo (ParamDecl _ n) = n
+        nodeInfo (AbstractParamDecl _ n) = n
+
+instance Pos ParamDecl where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode MemberDecl where
+        nodeInfo (MemberDecl _ _ n) = n
+        nodeInfo (AnonBitField _ _ n) = n
+
+instance Pos MemberDecl where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode TypeDef where
+        nodeInfo (TypeDef _ _ _ n) = n
+
+instance Pos TypeDef where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode TypeDefRef where
+        nodeInfo (TypeDefRef _ _ n) = n
+
+instance Pos TypeDefRef where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode CompTypeRef where
+        nodeInfo (CompTypeRef _ _ n) = n
+
+instance Pos CompTypeRef where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode EnumTypeRef where
+        nodeInfo (EnumTypeRef _ n) = n
+
+instance Pos EnumTypeRef where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode CompType where
+        nodeInfo (CompType _ _ _ _ n) = n
+
+instance Pos CompType where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode EnumType where
+        nodeInfo (EnumType _ _ _ n) = n
+
+instance Pos EnumType where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode Enumerator where
+        nodeInfo (Enumerator _ _ _ n) = n
+
+instance Pos Enumerator where
+        posOf x = posOf (nodeInfo x)
+
+
+instance CNode Attr where
+        nodeInfo (Attr _ _ n) = n
+
+instance Pos Attr where
+        posOf x = posOf (nodeInfo x)
+-- GENERATED STOP
