@@ -1,5 +1,5 @@
 {-# LANGUAGE MultiParamTypeClasses, TypeSynonymInstances, FlexibleContexts,FlexibleInstances,
-             PatternSignatures, PatternGuards, RankNTypes, ScopedTypeVariables #-}
+             PatternGuards, RankNTypes, ScopedTypeVariables #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Language.C.Analysis.TravMonad
@@ -66,7 +66,8 @@ import qualified Language.C.Analysis.DefTable as ST
 
 import Data.IntMap (insert, lookup)
 import Data.Maybe
-import Control.Monad(liftM)
+import Control.Applicative (Applicative(..))
+import Control.Monad (liftM, ap)
 import Prelude hiding (lookup)
 
 class (Monad m) => MonadName m where
@@ -404,6 +405,13 @@ withExtDeclHandler action handler =
     do modify $ \st -> st { doHandleExtDecl = handler }
        action
 
+instance Functor (Trav s) where
+    fmap = liftM
+
+instance Applicative (Trav s) where
+    pure  = return
+    (<*>) = ap
+
 instance Monad (Trav s) where
     return x  = Trav (\s -> Right (x,s))
     m >>= k   = Trav (\s -> case unTrav m s of
