@@ -46,6 +46,8 @@ import Language.C.Data.Name    (Name)
 import Language.C.Data.Ident    (Ident)
 import Language.C.Parser.Tokens (CToken(CTokEof))
 
+import Control.Applicative (Applicative(..))
+import Control.Monad (liftM, ap)
 import Data.Set  (Set)
 import qualified Data.Set as Set (fromList, insert, member, delete)
 
@@ -70,6 +72,13 @@ data PState = PState {
 
 newtype P a = P { unP :: PState -> ParseResult a }
 
+instance Functor P where
+  fmap = liftM
+
+instance Applicative P where
+  pure = return
+  (<*>) = ap
+
 instance Monad P where
   return = returnP
   (>>=) = thenP
